function gadget:GetInfo()
	return {
		name    = "Starcrap",
		desc    = "Implements neutral crap to interact with",
		author  = "Sprung",
		date    = "1/10/12",
		license = "PD",
		layer   = 10,
		enabled = true,
	}
end

local mapConfig = include("LuaRules/Configs/sc_map_def.lua")
if (not mapConfig) then mapConfig = include("LuaRules/Configs/sc_map_def.lua") end

if (not mapConfig) then return else
	local spGH = Spring.GetGroundHeight
	for i = 1, #mapConfig.towers do
		mapConfig.towers[i].draw = true
		mapConfig.towers[i][5] = mapConfig.towers[i][3]
		mapConfig.towers[i][3] = mapConfig.towers[i][2]
		mapConfig.towers[i][2] = spGH(mapConfig.towers[i][1], mapConfig.towers[i][3])
	end
	for i = 1, #mapConfig.rocks do
		mapConfig.rocks[i][4] = mapConfig.rocks[i][3]
		mapConfig.rocks[i][3] = mapConfig.rocks[i][2]
		mapConfig.rocks[i][2] = spGH(mapConfig.rocks[i][1], mapConfig.rocks[i][3])
	end
end

if (gadgetHandler:IsSyncedCode()) then
	local spCU = Spring.CreateUnit
	local neutral_team = Spring.GetGaiaTeamID()
	local _,_,_,_,_,neutral_allyteam = Spring.GetTeamInfo(neutral_team)

	local getAlly = Spring.GetUnitNearestAlly
	local getEnemy = Spring.GetUnitNearestEnemy
	local getCyl = Spring.GetUnitsInCylinder
	local getPos = Spring.GetUnitPosition
	local giev = Spring.TransferUnit
	local getTeam = Spring.GetUnitTeam
	local getAllyTeam = Spring.GetUnitAllyTeam
	local getAllyList = Spring.GetTeamList
	local stun_z_mlotka = Spring.GetUnitIsStunned
	local getMovetype = Spring.GetUnitMoveTypeData
	local callScript = Spring.UnitScript.CallAsUnit
	local SendToUnsync = SendToUnsynced
	local neutralize = Spring.SetUnitNeutral

	local xelnagas = {}
	local rocks = {}

	function gadget:GamePreload()
		for i = 1, #mapConfig.towers do
			local uid = spCU(mapConfig.towers[i][5], mapConfig.towers[i][1], mapConfig.towers[i][2], mapConfig.towers[i][3], 0, neutral_team)
			neutralize(uid, true)
			Spring.SetUnitAlwaysVisible(uid, true) -- brief visiblity for a ghost site to appear
			xelnagas[uid] = i
		end
		for i = 1, #mapConfig.rocks do
			local uid = spCU(mapConfig.rocks[i][4], mapConfig.rocks[i][1], mapConfig.rocks[i][2], mapConfig.rocks[i][3], 0, neutral_team)
			neutralize(uid, true)
			Spring.SetUnitAlwaysVisible(uid, true)
			rocks[uid] = true
		end
	end

	function gadget:AllowWeaponTarget(unitID, targetID, attackerWeaponNum, attackerWeaponDefID, defPriority)
		if(xelnagas[targetID]) then
			return false, defPriority
		else
			return true, defPriority
		end
	end

	function gadget:AllowCommand(unitID, unitDefID, unitTeam, cmdID, cmdParams, cmdOptions, cmdTag)
		if ((cmdID == CMD.ATTACK or cmdID == CMD.MANUALFIRE) and #cmdParams == 1) then
			local targetID = cmdParams[1]
			if(targetID and xelnagas[targetID]) then
				return false
			end
		end
		return true
	end
	
	function gadget:UnitPreDamaged(unitID, unitDefID, unitTeam, damage)
		if (unitID and xelnagas[unitID]) then
			return 0 -- can't touch this!
		else
			return damage
		end
	end

	function gadget:UnitDestroyed(uid, udef, team)
		if(uid) then
			if(xelnagas[uid]) then
				_G.XelTowerPwned = xelnagas[uid]
				SendToUnsync("XelTowerPwned")
				_G.XelTowerPwned = nil
				xelnagas[uid] = nil
			elseif(rocks and rocks[uid]) then
				rocks[uid] = nil
			end
		end
	end

	function gadget:GameFrame(n)
		if(n == 30) then
			for id,_ in pairs(xelnagas) do
				Spring.SetUnitAlwaysVisible(id, false)
			end
			for id,_ in pairs(rocks) do
				Spring.SetUnitAlwaysVisible(id, false)
			end
			rocks = nil
		end
		if (n % 30 == 5) then
			for id, data_id in pairs(xelnagas) do
				local x,_,z = getPos(id)
				local nubs = getCyl(x, z, mapConfig.towers[data_id][4])
				local counts = {}
				local ally_of_id = getAllyTeam(id)
				counts[ally_of_id] = (counts[ally_of_id] or 0) - 1
				for i = 1, #nubs do
					local nub = nubs[i]
					if(not stun_z_mlotka(nub) and getMovetype(nub).name ~= "airplane") then
						local allied = getAllyTeam(nub)
						counts[allied] = (counts[allied] or 0) + 1
					end
				end
				local winnar = nil
				local fail = false
				for team, unitcount in pairs(counts) do -- could probably be done in constant time, but meh
					if(team ~= neutral_allyteam and unitcount > 0) then
						if(not winnar) then
							winnar = team
						else
							fail = true
							break
						end
					end
				end
				if((not winnar) or fail) then
					if (neutral_team ~= ally_of_id) then
						if (GG.shareunits) then -- bypass the "dont share to enemy" thingy from ZK
							GG.shareunits[id] = true
						end
						giev(id, neutral_team, true)
						neutralize(id, true)
						local func = Spring.UnitScript.GetScriptEnv(id)
						if (func) then
							func = func.script.Deactivate
							if (func) then
								Spring.UnitScript.CallAsUnit(id,func, {}) -- call Deactivate(), if unit has one
							end
						end
					end
				elseif(winnar ~= ally_of_id) then
					if (GG.shareunits) then
						GG.shareunits[id] = true
					end
					giev(id, getAllyList(winnar)[1], true)
					neutralize(id, true)
					local func = Spring.UnitScript.GetScriptEnv(id)
					if (func) then
						func = func.script.Activate
						if (func) then
							Spring.UnitScript.CallAsUnit(id,func, {})
						end
					end
				end
			end
		end
	end
else -- un/sync
	local glColor = gl.Color
	local glDrawGroundCircle = gl.DrawGroundCircle

	function gadget:Initialize()
		gadgetHandler:AddSyncAction("XelTowerPwned", function() mapConfig.towers[SYNCED.XelTowerPwned].draw = false end)
	end

	function gadget:DrawWorldPreUnit()
		glColor(1.0, 1.0, 1.0, 0.4)
		for i = 1, #mapConfig.towers do
			if(mapConfig.towers[i].draw) then glDrawGroundCircle(mapConfig.towers[i][1], mapConfig.towers[i][2], mapConfig.towers[i][3], mapConfig.towers[i][4], 32) end
		end
		glColor(1,1,1,1)
	end
end