function widget:GetInfo()
	return {
		name = "Mineralz", -- acuelly is xray shader
		desc = "Renders minerals",
		author = "trepan, sprung",
		date = "15/7/07 - 15/1/13",
		license = "GNU GPL, v2 or later",
		layer = 0,
		enabled = true,
	}
end

Spring.Echo("LuaRules/Configs/MetalSpots/" .. (Game.mapName or "") .. ".lua !!")

local minerals_FDefID = FeatureDefNames["minerals_feature"]
if (minerals_FDefID) then minerals_FDefID = minerals_FDefID.id end

if (not minerals_FDefID) then return end

local spGetFeatureDefID = Spring.GetFeatureDefID

	local GL_ONE                 = GL.ONE
	local GL_ONE_MINUS_SRC_ALPHA = GL.ONE_MINUS_SRC_ALPHA
	local GL_SRC_ALPHA           = GL.SRC_ALPHA
	local glBlending             = gl.Blending
	local glColor                = gl.Color
	local glCreateShader         = gl.CreateShader
	local glDeleteShader         = gl.DeleteShader
	local glDepthTest            = gl.DepthTest
	local glUnit                 = gl.Unit
	local glFeature              = gl.Feature
	local glGetShaderLog         = gl.GetShaderLog
	local glPolygonOffset        = gl.PolygonOffset
	local glSmoothing            = gl.Smoothing
	local glUseShader            = gl.UseShader
	local spEcho                 = Spring.Echo
	local spGetAllFeatures   = Spring.GetAllFeatures
	local spGetVisibleUnits      = Spring.GetVisibleUnits
	local spGetUnitDefID         = Spring.GetUnitDefID

	if (not glCreateShader) then
		spEcho("Hardware is incompatible with Minerals shader requirements")
		return false
	end

	local shader

	function gadget:Shutdown()
		glDeleteShader(shader)
	end

	function gadget:Initialize()

		shader = glCreateShader({

			uniform = {
				edgeExponent = 3.5,
			},

			vertex = [[
				// Application to vertex shader
				varying vec3 normal;
				varying vec3 eyeVec;
				varying vec3 color;
				uniform mat4 camera;
				uniform mat4 caminv;

				void main()
				{
					vec4 P = gl_ModelViewMatrix * gl_Vertex;
					eyeVec = P.xyz;
					normal  = gl_NormalMatrix * gl_Normal;
					color = gl_Color.rgb;
					gl_Position = gl_ProjectionMatrix * P;
				}
				]],  

			fragment = [[
				varying vec3 normal;
				varying vec3 eyeVec;
				varying vec3 color;

				uniform float edgeExponent;

				void main()
				{
					float opac = dot(normalize(normal), normalize(eyeVec));
					opac = 1.0 - abs(opac);
					opac = 0.5 + (0.4 * opac); 
					opac = pow(opac, edgeExponent);

					gl_FragColor.rgb = color;
					gl_FragColor.a = opac;
				}
			]],
		})

		if (shader == nil) then
			spEcho(glGetShaderLog())
			spEcho("Xray shader compilation failed.")
			gadgetHandler:RemoveGadget()
		end
	end

	function widget:DrawWorld()
		glSmoothing(nil, nil, true)
		glUseShader(shader)
		glDepthTest(true)
		glBlending(GL_SRC_ALPHA, GL_ONE)
		glPolygonOffset(-2, -2)
		glColor(0, 0.6, 1)

		if (minerals_FDefID) then
			for _, fID in ipairs(spGetAllFeatures()) do
				if (spGetFeatureDefID(fID) == minerals_FDefID) then glFeature(fID, true) end
			end
		end

		glPolygonOffset(false)
		glBlending(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
		glDepthTest(false)
		glUseShader(0)
		glColor(1, 1, 1, 1)
		glSmoothing(nil, nil, false)
	end